using System;
using System.Data;
using System.Data.SqlClient;

namespace PrikladyZive59
{
	/// <summary>
	/// Ukazka pouziti izolacnich urovni transakci
	/// </summary>
	public class IsolationLevelExam
	{
		static internal void Run()
		{
			//vytvoreni spojeni pro transakce
			SqlConnection lConn1 = new SqlConnection(@"data source='dellak\sql1';initial catalog='zive'; integrated security='sspi'");
			SqlConnection lConn2 = new SqlConnection(@"data source='dellak\sql1';initial catalog='zive'; integrated security='sspi'");
			SqlCommand lInsertComm = lConn1.CreateCommand();
			SqlCommand lSelectComm = lConn2.CreateCommand();
			SqlCommand lDeleteComm = lConn1.CreateCommand();
			//prikaz pro vytvoreni zaznamu
			lInsertComm.CommandText = "insert into employee (FirstName, SurName, BirthDate) values('Jan', 'Novak', @BirthDate)";
			//prikaz pro vybrani zaznamu
			lSelectComm.CommandText = "select * from employee where SurName='Novak'";
			//prikaz pro odstraneni zaznamu
			lDeleteComm.CommandText = "delete from employee where SurName='Novak'";

			lInsertComm.Parameters.Add("@BirthDate", SqlDbType.DateTime).Value = DateTime.Now;

			SqlTransaction lTrans1 = null;
			SqlTransaction lTrans2 = null;
			try
			{
				//otevreni obou spojeni
				lConn1.Open();
				lConn2.Open();

				//zapoceti obou transakci
				lTrans1 = lConn1.BeginTransaction();
				//prikazy v teto transakci mohou cist i neodevzdana data jinymi transakcemi
				lTrans2 = lConn2.BeginTransaction(IsolationLevel.ReadUncommitted);

				lInsertComm.Transaction = lTrans1;
				lSelectComm.Transaction = lTrans2;
				lDeleteComm.Transaction = lTrans1;
				
				lInsertComm.ExecuteNonQuery();
				Console.WriteLine("Transakce 1 vlozila zaznam");

				//bude zobrazen i zaznam pridany druhou transakci, prestoze
				//nebyla transakce potvrzena
				Console.WriteLine("Transakce 2 ziskala tato data :");
				ShowResult(lSelectComm.ExecuteReader());
				
				lDeleteComm.ExecuteNonQuery();
				Console.WriteLine("Transakce 1 odstranila zaznam");
				
				lTrans1.Commit();
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex);
				if (lTrans1 != null)
				{
					lTrans1.Rollback();
				}
			}
			finally
			{
				if (lConn1 != null)
				{
					lConn1.Close();
				}
				if (lConn2 != null)
				{
					lConn2.Close();
				}
			}
			
		}

		/// <summary>
		/// Vypise vysledkovou sadu
		/// </summary>
		/// <param name="reader">reader vysledkove sady</param>
		static void ShowResult(IDataReader reader)
		{
			try
			{
				while (reader.Read())
				{
					for (int i = 0; i < reader.FieldCount; i++)
					{
						Console.Write("{0} - ", reader[i]);
					}
					Console.WriteLine();
				}
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex);
			}
			finally
			{
				if (reader != null)
				{
					reader.Close();
				}
			}
		}
	}
}
